/*
   This program uses a small "Domain Specific Language" (DSL) to
   solve the problem of instantiating binary trees. Binary trees
   are represented by strings in a simple BTree language and a
   "parser" reads these strings and builds the appropriate binary tree.

                 a
               /   \
              /     \
            b         c
          /   \        \
         /     \        \
        d       e        f
                        /
                       /
                      g
                       \
                        \
                         h


   This program assumes that you have installed Graphviz, which includes
   the dot.exe program. You can download Graphviz and dot.exe from the
   following URL. Unzip the zip file to your C:\ drive.

     https://graphviz.gitlab.io/_pages/Download/windows/graphviz-2.38.zip
*/

public class BTree_Language_Example_1
{
   public static void main(String[] args)
   {
      // The string that represents the above btree.
      String btreeLanguage = "( a ( b d e ) ( c () ( f ( g () h ) () ) ) )";

      // Build the BTree data structure from its string representation.
      BTree btree = BuildBTree.buildBTree( btreeLanguage );

      // Print the BTree using the BTree's toString() method.
      System.out.println("Binary tree expression is " + btree + "\n" );

      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing (and building)
      // the tree, so a "pretty-printer" is sometimes called an "unparser".
      System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );

      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      // We should get "a b d e c f g h".
      System.out.println( " Pre-order traversal = " + Traverse.preOrder( btree ) + "\n" );
      // We should get "d b e a c g h f".
      System.out.println( "  In-order traversal = " + Traverse.inOrder( btree ) + "\n" );
      // We should get "d e b h g f c a".
      System.out.println( "Post-order traversal = " + Traverse.postOrder( btree ) + "\n" );


      // Create dot and png files from the tree.
      if(true)
      try
      {
         // Create the (empty) dot file.
         String baseName = "Example_2";
         java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot"));
         // Put dot commands into the dot file
         out.println( BTree2dot.btree2dot(btree) + "\n" );
         out.close();

         // Create a command line for running dot.exe.
         String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                         "-Tpng",
                         baseName + ".dot",
                         "-o",
                         baseName + ".png"};
         // Execute the command line.
         java.lang.Runtime.getRuntime().exec(cmd);
      }
      catch (Exception e)
      {
         System.out.println( e );
      }
   }
}
